package sandbox;

import processing.core.*;

/*
 * Copyright (c) 2007 Karsten Schmidt
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * http://creativecommons.org/licenses/LGPL/2.1/
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */


/**
 * Axis-aligned bounding box with basic intersection features for Ray, AABB and
 * Sphere classes.
 */

public class AABB extends PVector {


    protected PVector extent;
    protected PVector min, max;

	
	
    /**
     * From toxiclibs:
     * Creates an independent copy of the passed in box
     * 
     * @param box
     */
    public AABB(AABB box) {
        this(box, box.getExtent());
    }


    /**
     * From toxiclibs:
     * Creates a new instance from center point and extent
     * 
     * @param pos
     * @param extent
     *            box dimensions (the box will be double the size in each
     *            direction)
     */
    public AABB(PVector pos, PVector extent) {
        super(pos.x, pos.y, pos.z);
        min = new PVector();
        max = new PVector();
        setExtent(extent.x, extent.y, extent.z);
    }

    public AABB copy() {
        return new AABB(this);
    }

    /**
     * From toxiclibs:
     * Returns the current box size as new PVector instance (updating this vector
     * will NOT update the box size! Use {@link #setExtent(PVector)} for those
     * purposes)
     * 
     * @return box size
     */
    public final PVector getExtent() {
        return extent.get();
    }

    public final PVector getMax() {
        return max.get();
    }

    public final PVector getMin() {
        return min.get();
    }

    
    /**
     * From toxiclibs:
     * Checks if the box contains the passed vector.
     * 
     * @param PVector
     *            vector to check
     * @return true, if the vector is inside the box
     */
    public boolean containsVector(PVector v) {
        return v.x > min.x && v.x < max.x
        		&& v.y > min.y && v.y < max.y
        		&& v.z > min.z && v.z < max.z; 
    }
    
    
    /**
     * From toxiclibs:
     * Checks if the box intersects the passed in one.
     * 
     * @param box
     *            box to check
     * @return true, if boxes overlap
     */
    public boolean intersectsBox(AABB box) {
        PVector t = PVector.sub(box, this);
        return Math.abs(t.x) <= (extent.x + box.extent.x)
                && Math.abs(t.y) <= (extent.y + box.extent.y)
                && Math.abs(t.z) <= (extent.z + box.extent.z);
    }

    /**
     * From toxiclibs:
     * @param c
     *            sphere centre
     * @param r
     *            sphere radius
     * @return true, if AABB intersects with sphere
     */
    public boolean intersectsSphere(PVector c, float r) {
        float s, d = 0;
        // find the square of the distance
        // from the sphere to the box
        if (c.x < min.x) {
            s = c.x - min.x;
            d += s * s;
        } else if (c.x > max.x) {
            s = c.x - max.x;
            d += s * s;
        }

        if (c.y < min.y) {
            s = c.y - min.y;
            d += s * s;
        } else if (c.y > max.y) {
            s = c.y - max.y;
            d += s * s;
        }

        if (c.z < min.z) {
            s = c.z - min.z;
            d += s * s;
        } else if (c.z > max.z) {
            s = c.z - max.z;
            d += s * s;
        }

        return d <= r * r;
    }


    /**
     * From toxiclibs:
     * Updates the size of the box and calls {@link #updateBounds()} immediately
     * 
     * @param float	x, y, z
     *            new box size
     * @return itself, for method chaining
     */
    public AABB setExtent(float x, float y, float z) {
        this.extent = new PVector(x, y, z);
        return updateBounds();
    }    
    
    
    /**
     * From toxiclibs:
     * Updates the size of the box and calls {@link #updateBounds()} immediately
     * 
     * @param extent
     *            new box size
     * @return itself, for method chaining
     */
    public AABB setExtent(PVector extent) {
        this.extent = new PVector(extent.x, extent.y, extent.z);
        return updateBounds();
    }


    /**
     * From toxiclibs:
     * Updates the min/max corner points of the box. MUST be called after moving
     * the box in space by manipulating the public x,y,z coordinates directly.
     * 
     * @return itself
     */
    public final AABB updateBounds() {
        // this is check is necessary for the constructor
        if (extent != null) {
            this.min.set(PVector.sub(this, extent));
            this.max.set(PVector.add(this, extent));
        }
        return this;
    }
}

